@@ -1,11 +1,14 @@
 import pandas as pd
 import numpy as np
 from pandas import read_csv
+from scipy.ndimage import uniform_filter1d
+import csv
 
 class Test:
 
-    def __init__(self, filename):
+    def __init__(self, filename, filter_size = 20):
         self.filename = filename
+        self.filter_size = filter_size
 
     def set_data_path(self, data_path):
         self.data_path = data_path
@@ -14,31 +17,48 @@ class Test:
         self.data_extension = data_extension
 
     def clear_file_name(self):
-        if '.' in self.filename and self.filename.split('.')[-1]=='.csv':
+        if '.' in self.filename and self.filename.split('.')[-1] == '.csv':
             self.filename_cleared = self.filename.split('.')[0]
         else:
             self.filename_cleared = self.filename
 
         self.filename_cleared = self.filename_cleared.rpartition('/')[-1]
 
-    def infer_metabolimeter(self):
+    def infer_metabolimeter(self, optional_data=[]):
 
         import chardet
 
         try:
-            if self.data_extension == '.csv':
+            if self.data_extension == '.csv' or self.data_extension == '.CSV':
                 df = read_csv(self.filename + self.data_extension)
+                if len(df.columns) == 1:
+                    df = read_csv(self.filename + self.data_extension, delimiter=";")
                 print('Just reading a csv file')
             if self.data_extension == '.txt':
                 df = read_csv(self.filename + self.data_extension, sep="\t", header=None, skiprows=3)
                 print('Just reading a txt file')
                 self.metabolimeter = 'vyiare'
+            if self.data_extension == '.xlsx' or self.data_extension == '.xls':
+                print('Attempting to read an Excel file')
+                df = pd.read_excel(self.filename + self.data_extension)
         except:
-            f = open(self.filename + self.data_extension, encoding="utf8", errors="ignore")
-            df = read_csv(f, header=0)
+            try:
+                f = open(self.filename + self.data_extension, encoding="utf8", errors="ignore")
+                df = read_csv(f, header=0)
+            except:
+                df = optional_data
 
         self.df = df
 
+        if 'Victorian Institute of Sport' in df.columns:
+            self.metabolimeter = 'VIS'
+
+        if 'New South Wales Institute of Sport' in df.columns:
+            self.metabolimeter = 'NSWIS'
+
+        if 'South Australian Sports Institute' in df.columns:
+            self.metabolimeter = 'SASI'
+
         if 'VO2_I' in df.columns:
             self.metabolimeter = 'generated_pyoxynet'
 
@@ -132,13 +152,93 @@ class Test:
                 print('Most probably we have an French file')
                 self.metabolimeter = 'mourot_COPD'
 
+            if 'RF' in df.columns and 't' in df.columns and 'VE/VO2' in df.columns and 'VE/VCO2' in df.columns and 'R' in df.columns:
+                print('Most probably we have an file coming from the Exercise Threshold App')
+                self.metabolimeter = 'exercise_threshold_app'
+
+            if " V'O2   " in df.columns and " V'CO2  " in df.columns:
+                print('Most probably we have an file coming from the Uni of Ess')
+                self.metabolimeter = 'uni_of_essex'
+
+            if "T(sec)" in df.columns and "PHASE" in df.columns:
+                print('Most probably we have an rowing file coming')
+                self.metabolimeter = 'rowing_ben'
+
         except:
             pass
 
     def load_file(self):
-        from . import utilities
 
+        from . import utilities
         df = self.df
+        filter_size = self.filter_size
+
+        if self.metabolimeter == 'SASI' or self.metabolimeter == 'NSWIS' or self.metabolimeter == 'VIS':
+
+            try:
+                starting_index = df[df['South Australian Sports Institute'] == 'min  '].index
+            except:
+                pass
+            try:
+                starting_index = df[df['New South Wales Institute of Sport'] == 'min  '].index
+            except:
+                pass
+            try:
+                starting_index = df[df['Victorian Institute of Sport'] == 'min  '].index
+            except:
+                pass
+            ventilatory_data = df[starting_index[0]+1:-1]
+
+            # print('Weight')
+            self.weight = 70
+            # print('Height')
+            self.height = 180
+            # print('Age')
+            self.age = 40
+            # print('Gender')
+            self.gender = 'M'
+
+            # print('Reading data')
+            n_rows = len(ventilatory_data.index)
+            # initialise variables
+            self.time = np.zeros((n_rows,), dtype=np.float32)
+            self.VO2 = np.zeros((n_rows,), dtype=np.float32)
+            self.VCO2 = np.zeros((n_rows,), dtype=np.float32)
+            self.HR = np.zeros((n_rows,), dtype=np.float32)
+            self.Rf = np.zeros((n_rows,), dtype=np.float32)
+            self.VE = np.zeros((n_rows,), dtype=np.float32)
+            self.PetO2 = np.zeros((n_rows,), dtype=np.float32)
+            self.PetCO2 = np.zeros((n_rows,), dtype=np.float32)
+            self.load = np.zeros((n_rows,), dtype=np.float32)
+
+            i = 0
+            max_time = 0
+            # Protection mechanism in place for "2-in-1" tests
+            # TODO: good for every kind of institution/metabolimeter TBH
+            for time_sec in ventilatory_data[ventilatory_data.columns[0]].values:
+                try:
+                    if len(time_sec.split(':')) > 2:
+                        time_stamp = utilities.get_sec(':'.join(time_sec.split(':')[:2]))
+                    else:
+                        time_stamp = utilities.get_sec(time_sec)
+
+                    if time_stamp > max_time & (time_stamp - max_time) < 20:
+                        max_time = time_stamp
+                        self.time[i] = time_stamp
+                        self.VO2[i] = float(ventilatory_data[ventilatory_data.columns[2]].values[i]) * 1000
+                        self.VCO2[i] = float(ventilatory_data[ventilatory_data.columns[4]].values[i]) * 1000
+                        self.HR[i] = 0
+                        self.load[i] = 0
+                        self.VE[i] = float(ventilatory_data[ventilatory_data.columns[1]].values[i])
+                        self.Rf[i] = float(ventilatory_data[ventilatory_data.columns[10]].values[i])
+                        self.PetO2[i] = float(ventilatory_data[ventilatory_data.columns[6]].values[i])
+                        self.PetCO2[i] = float(ventilatory_data[ventilatory_data.columns[5]].values[i])
+                        i += 1
+                    else:
+                        break
+
+                except:
+                    pass
 
         if self.metabolimeter == 'vintus':
 
@@ -347,7 +447,7 @@ class Test:
 
         if self.metabolimeter == 'cortex':
 
-            df = read_csv(self.filename + self.data_extension)
+            df = self.df
 
             try:
                 starting_index = df.loc[df['Unnamed: 0'] == 't'].index
@@ -453,26 +553,32 @@ class Test:
 
                 i += 1
 
-            # self.time = self.time - self.time[0]
-            if use_WR:
-                VT1_index = np.where(self.load == int(WRVT1))[0][0]
-                try:
-                    VT2_index = np.where(self.load == int(WRVT2))[0][0]
-                except:
-                    VT2_index = -1
-            elif use_HR:
-                VT1_index = np.where(self.HR == int(HRVT1))[0][0]
-                try:
-                    VT2_index = np.where(self.HR == int(HRVT2))[0][0]
-                except:
-                    VT2_index = -1
+            VT1_index = 0
+            VT2_index = 0
+
+            try:
+                # self.time = self.time - self.time[0]
+                if use_WR:
+                    VT1_index = np.where(self.load == int(WRVT1))[0][0]
+                    try:
+                        VT2_index = np.where(self.load == int(WRVT2))[0][0]
+                    except:
+                        VT2_index = -1
+                elif use_HR:
+                    VT1_index = np.where(self.HR == int(HRVT1))[0][0]
+                    try:
+                        VT2_index = np.where(self.HR == int(HRVT2))[0][0]
+                    except:
+                        VT2_index = -1
+            except:
+                pass
 
             self.VT1 = self.time[VT1_index]
             self.VT2 = self.time[VT2_index]
 
         if self.metabolimeter == 'unisbz':
 
-            df = read_csv(self.filename + self.data_extension)
+            df = self.df
 
             self.age = float(0)
             # print('Gender')
@@ -548,6 +654,7 @@ class Test:
 
             # get the time in seconds
             # convert time data from "HH:MM:SS" to seconds
+            df['t'] = df['t'].astype(str)
             for i in df.index[2:n_rows]:
                 try:
                     if df.t[0] == 's':
@@ -562,9 +669,12 @@ class Test:
                 self.VO2[i - 2] = float(df.VO2.values[i])
                 self.VCO2[i - 2] = float(df.VCO2.values[i])
                 try:
-                    self.HR[i - 2] = float(df.HR.values[i])
+                    try:
+                        self.HR[i - 2] = float(df.HR.values[i])
+                    except:
+                        self.HR[i - 2] = float(df.HF.values[i])
                 except:
-                    self.HR[i - 2] = float(df.HF.values[i])
+                    self.HR[i - 2] = 0
                 try:
                     self.Rf[i - 2] = float(df.Rf.values[i])
                 except:
@@ -580,7 +690,8 @@ class Test:
                 self.PetCO2[i - 2] = float(df.PetCO2.values[i])
 
         if self.metabolimeter == 'cortex_bruce':
-            df = read_csv(self.filename + self.data_extension)
+
+            df = self.df
             print('Reading data Bruce file!!!!')
 
             starting_index = df.loc[df[df.columns[0]] == 'Messzeit'].index+2
@@ -641,7 +752,8 @@ class Test:
                     self.PetCO2[i - starting_index[0]] = np.nan
 
         if self.metabolimeter == 'cortex_bruce_2':
-            df = read_csv(self.filename + self.data_extension)
+
+            df = self.df
             print('Reading data Bruce file type 2!!!!')
 
             starting_index = df.loc[df[df.columns[0]] == 't'].index+2
@@ -702,7 +814,8 @@ class Test:
                     self.PetCO2[i - starting_index[0]] = np.nan
 
         if self.metabolimeter == 'mourot':
-            df = read_csv(self.filename + self.data_extension)
+
+            df = self.df
             # print('Reading data Mourot file!!!!')
 
             starting_index = df.loc[df[df.columns[0]] == 'Temps'].index+2
@@ -778,8 +891,8 @@ class Test:
 
         if self.metabolimeter == 'mourot_COPD':
 
-            df = read_csv(self.filename + self.data_extension)
-            # print('Reading data Mourot file with no age nor BMI data !!!!')
+            df = self.df
+            print('Reading data Mourot file with no age nor BMI data !!!!')
 
             starting_index = [1]
             n_rows = df[df.columns[0]].last_valid_index() - 3
@@ -834,7 +947,7 @@ class Test:
 
         if self.metabolimeter == 'low':
 
-            df = read_csv(self.filename + self.data_extension)
+            df = self.df
 
             # print('Reading age')
             self.age = float(30)
@@ -875,7 +988,7 @@ class Test:
 
         if self.metabolimeter == 'VO2Master':
 
-            df = read_csv(self.filename + self.data_extension)
+            df = self.df
 
             # print('Reading age')
             self.age = float(30)
@@ -916,18 +1029,18 @@ class Test:
 
         if self.metabolimeter == 'centro-monzino':
 
-            df = read_csv(self.filename + self.data_extension)
+            df = self.df
 
             # print('Reading age')
             self.age = float(30)
-            # print('Gender')
+            # print('Gender')
             self.gender = 'N'
             # print('Weight')
             self.weight = float(50)
             # print('Height')
             self.height = float(180)
 
-            # print('Reading data')
+            # print('Reading data')
             n_rows = len(df.index)
             self.time = np.zeros((n_rows - 2,), dtype=np.float32)
             self.VO2 = np.zeros((n_rows - 2,), dtype=np.float32)
@@ -1033,6 +1146,124 @@ class Test:
             # print('Height')
             self.height = df.height.mean()
 
+        if self.metabolimeter == 'exercise_threshold_app':
+            self.time = df.t
+            self.VO2 = df.VO2
+            self.VCO2 = df.VCO2
+            self.HR = df.HR
+            self.Rf = df.RF
+            self.VE = df.VE
+            self.PetO2 = df.PetO2
+            self.PetCO2 = df.PetCO2
+            # default to 45
+            self.age = 45
+            # default to N
+            self.gender = 'N'
+            # print('Weight')
+            # default to 70
+            self.weight = 70
+            # print('Height')
+            # default to 170
+            self.height = 170
+
+        if self.metabolimeter == 'uni_of_essex':
+
+            # print('Reading age')
+            self.age = float(30)
+            # print('Gender')
+            self.gender = 'N'
+            # print('Weight')
+            self.weight = float(60)
+            # print('Height')
+            self.height = float(170)
+
+            # print('Reading data')
+            n_rows = len(df.index)
+            self.time = np.zeros((n_rows - 2,), dtype=np.float32)
+            self.VO2 = np.zeros((n_rows - 2,), dtype=np.float32)
+            self.VCO2 = np.zeros((n_rows - 2,), dtype=np.float32)
+            self.HR = np.zeros((n_rows - 2,), dtype=np.float32)
+            self.Rf = np.zeros((n_rows - 2,), dtype=np.float32)
+            self.VE = np.zeros((n_rows - 2,), dtype=np.float32)
+            self.PetO2 = np.zeros((n_rows - 2,), dtype=np.float32)
+            self.PetCO2 = np.zeros((n_rows - 2,), dtype=np.float32)
+            self.load = np.zeros((n_rows - 2,), dtype=np.float32)
+
+            # get the time in seconds
+            # convert time data from "HH:MM:SS" to seconds
+            for i in df.index[2:n_rows]:
+                try:
+                    self.time[i - 2] = utilities.get_sec(df[df.columns[0]].values[i])
+                    self.VO2[i - 2] = float(df[df.columns[1]].values[i])
+                    self.VCO2[i - 2] = float(df[df.columns[2]].values[i])
+                    self.VE[i - 2] = float(df[df.columns[3]].values[i])
+                    self.PetO2[i - 2] = float(df[df.columns[4]].values[i]) * 7.50062
+                    self.PetCO2[i - 2] = float(df[df.columns[5]].values[i]) * 7.50062
+                    self.Rf[i - 2] = float(df[df.columns[9]].values[i])
+                except:
+                    self.time[i - 2] = np.nan
+                    self.VO2[i - 2] = np.nan
+                    self.VCO2[i - 2] = np.nan
+                    self.VE[i - 2] = np.nan
+                    self.HR[i - 2] = np.nan
+                    self.PetO2[i - 2] = np.nan
+                    self.PetCO2[i - 2] = np.nan
+                    self.Rf[i - 2] = np.nan
+
+        if self.metabolimeter == 'rowing_ben':
+
+            df = self.df
+            print('Reading data rowing file with no age nor BMI data !!!!')
+
+            starting_index = [1]
+            n_rows = df[df.columns[0]].last_valid_index() - 3
+
+            # print('Reading age')
+            my_array = np.where(df[df.columns[6]].notnull() == True, df.index, 0)
+            res = next(x for x, val in enumerate(my_array) if val > 0.6)
+
+            self.age = float(0)
+            # print('Gender')
+            self.gender = 'N'
+            # print('Weight')
+            self.weight = float(0)
+            # print('Height')
+            self.height = float(0)
+
+            self.time = np.zeros((n_rows - starting_index[0],), dtype=np.float32)
+            self.VO2 = np.zeros((n_rows - starting_index[0],), dtype=np.float32)
+            self.VCO2 = np.zeros((n_rows - starting_index[0],), dtype=np.float32)
+            self.HR = np.zeros((n_rows - starting_index[0],), dtype=np.float32)
+            self.VE = np.zeros((n_rows - starting_index[0],), dtype=np.float32)
+            self.PetO2 = np.zeros((n_rows - starting_index[0],), dtype=np.float32)
+            self.PetCO2 = np.zeros((n_rows - starting_index[0],), dtype=np.float32)
+            self.load = np.zeros((n_rows - starting_index[0],), dtype=np.float32)
+            self.Rf = np.zeros((n_rows - starting_index[0],), dtype=np.float32)
+
+            for i in np.arange(starting_index[0],n_rows):
+                try:
+                    self.time[i - starting_index[0]] = df[df.columns[0]].values[i]
+                except:
+                    self.time[i - starting_index[0]] = np.nan
+                try:
+                    # print(i)
+                    self.VO2[i - starting_index[0]] = float(df[df.columns[3]].values[i])
+                    self.VCO2[i - starting_index[0]] = float(df[df.columns[4]].values[i])
+                    self.VE[i - starting_index[0]] = float(df[df.columns[6]].values[i])
+                    self.HR[i - starting_index[0]] = float(df[df.columns[2]].values[i])
+                    self.PetO2[i - starting_index[0]] = float(df[df.columns[11]].values[i])
+                    self.PetCO2[i - starting_index[0]] = float(df[df.columns[12]].values[i])
+                    self.Rf[i - starting_index[0]] = float(df[df.columns[16]].values[i])
+                except:
+                    self.time[i - starting_index[0]] = np.nan
+                    self.VO2[i - starting_index[0]] = np.nan
+                    self.VCO2[i - starting_index[0]] = np.nan
+                    self.VE[i - starting_index[0]] = np.nan
+                    self.HR[i - starting_index[0]] = np.nan
+                    self.PetO2[i - starting_index[0]] = np.nan
+                    self.PetCO2[i - starting_index[0]] = np.nan
+                    self.Rf[i - starting_index[0]] = np.nan
+
         self.time = np.nan_to_num(self.time)
         self.VO2 = np.nan_to_num(self.VO2)
         self.VCO2 = np.nan_to_num(self.VCO2)
@@ -1042,21 +1273,22 @@ class Test:
         self.PetO2 = np.nan_to_num(self.PetO2)
         self.PetCO2 = np.nan_to_num(self.PetCO2)
 
+        # Rolling averages
         if self.metabolimeter != 'low':
 
-            self.Rf_F = utilities.optimal_filter(self.time, self.Rf, 200)
-            self.PetO2_F = utilities.optimal_filter(self.time, self.PetO2, 200)
-            self.PetCO2_F = utilities.optimal_filter(self.time, self.PetCO2, 200)
+            self.Rf_F = uniform_filter1d(self.Rf, size=filter_size)
+            self.PetO2_F = uniform_filter1d(self.PetO2, size=filter_size)
+            self.PetCO2_F = uniform_filter1d(self.PetCO2, size=filter_size)
 
         if self.metabolimeter == 'low':
             self.Rf_F = np.zeros((n_rows - 2,), dtype=np.float32)
             self.PetO2_F = np.zeros((n_rows - 2,), dtype=np.float32)
             self.PetCO2_F = np.zeros((n_rows - 2,), dtype=np.float32)
 
-        self.VO2_F = utilities.optimal_filter(self.time, self.VO2, 200)
-        self.VCO2_F = utilities.optimal_filter(self.time, self.VCO2, 200)
-        self.HR_F = utilities.optimal_filter(self.time, self.HR, 200)
-        self.VE_F = utilities.optimal_filter(self.time, self.VE, 200)
+        self.VO2_F = uniform_filter1d(self.VO2, size=filter_size)
+        self.VCO2_F = uniform_filter1d(self.VCO2, size=filter_size)
+        self.HR_F = uniform_filter1d(self.HR, size=filter_size)
+        self.VE_F = uniform_filter1d(self.VE, size=filter_size)
 
         # maximal VE value (cutting after)
         max_VE_idx = max([(v, i) for i, v in enumerate(self.VE_F)])[1]
@@ -1087,6 +1319,31 @@ class Test:
             print("The name of the participant is: " + self.name + '.\n'    "She is " + str(
                 self.age) + ' years old.\nWeight: ' + str(self.weight) + ' kg\nHeight: ' + str(self.height) + ' cm')
 
+    def create_raw_data_frame(self):
+
+        self.raw_data_frame = pd.DataFrame()
+
+        self.raw_data_frame['time'] = self.time
+        self.raw_data_frame['VO2'] = self.VO2
+        self.raw_data_frame['VCO2'] = self.VCO2
+        self.raw_data_frame['VE'] = self.VE
+        self.raw_data_frame['HR'] = self.HR
+        self.raw_data_frame['RF'] = self.Rf
+        self.raw_data_frame['PetO2'] = self.PetO2
+        self.raw_data_frame['PetCO2'] = self.PetCO2
+
+        # variables normalised on VO2
+        self.raw_data_frame['VEVO2'] = self.VE / self.VO2
+        self.raw_data_frame['VCO2VO2'] = self.VCO2 / self.VO2
+        self.raw_data_frame['PetO2VO2'] = self.PetO2 / self.VO2
+        self.raw_data_frame['PetCO2VO2'] = self.PetCO2 / self.VO2
+
+        if self.metabolimeter == 'VO2Master':
+            # putting to zero
+            self.raw_data_frame['VEVCO2'] = self.VCO2
+        else:
+            self.raw_data_frame['VEVCO2'] = self.VE / self.VCO2
+
     def create_data_frame(self):
 
         self.data_frame = pd.DataFrame()
@@ -1111,6 +1368,7 @@ class Test:
             self.data_frame['VEVCO2_I'] = self.VCO2_I
         else:
             self.data_frame['VEVCO2_I'] = self.VE_I / self.VCO2_I
+            
         self.data_frame['age'] = self.age
 
         # height in cm
